#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "NebulaDevice.h"
#include "IOTCDevice.h"
#include "AVAPIs.h"
#include "disposable_params.h"
#include <windows.h>

static char gUdid[MAX_UDID_LENGTH + 1] = {0};
static char gPinCode[MAX_PIN_CODE_LENGTH + 1] = {0};
static char gSecretId[MAX_NEBULA_SECRETID_LENGTH + 1] = {0};
static char gPsk[MAX_NEBULA_PSK_LENGTH + 1] = {0};
static unsigned int gBindAbort = 0;

void WINAPI identity_handle(NebulaDeviceCtx *device, const char *identity, char *psk, unsigned int psk_size)
{
    if(psk_size > strlen(gPsk))
        strcpy(psk, gPsk);
}

int WINAPI command_handle(NebulaDeviceCtx *device, const char *identity, const char *func, const TUTKJsonObject *json_args, TUTKJsonObject **json_response)
{
    printf("[command_handle] %s\n", func);

    char resp[100]={0};
    int ret = 0, night_mode = 0;

    TUTKJsonObject *response;
    if (strcmp("getNightMode", func) == 0) {
        // The profile or document might describe getNightMode as
        // { 
        //   "func":"getNightMode",
        //   "return": {
        //     "value":"Int"
        //   }
        // }
        // When the value of night mode is 10, please make a JSON response as { "value": 10 }
        // There is no need to add key "content" here    

        sprintf(resp, "{\"value\":%d}", night_mode);
        printf("command_handle: send response[%s]\n", resp);

        ret = TUTK_Json_Obj_Create_From_String(resp, &response);

        if (response == NULL) {
            printf("command_handle: Invalid response error ret [%d]!!\n", ret);
            return 400;
        }

        *json_response = response;
        return 200;
    } else {
        strcpy(resp, "{\"error\":\"unknow func\"}");
        ret = TUTK_Json_Obj_Create_From_String(resp, json_response);
        printf("[%s] unknown func : %s[%d]\n", __func__, func, ret);
    }
    return 400;
}


int WINAPI settings_change_handle(NebulaDeviceCtx* device, const char* settings)
{
    printf("in settings_change_handle\n");
    printf("settings[%s]\n", settings);
    return 0;
}

int WINAPI device_login_state_handle(NebulaDeviceCtx* device, NebulaDeviceLoginState state)
{
    printf("in device_login_state_handle\n");
    printf("login state[%d]\n", state);
    return 0;
}

int main(int argc, char *argv[])
{
	int ret;
    FILE *fp;
    NebulaDeviceCtx *device_ctx;

	if(argc != 4)
	{
		printf("%s <LicenseKey> <UDID> <Profile>\n",argv[0]);
		return -1;
	}

    char *license_key = argv[1];
    ret = TUTK_SDK_Set_License_Key(license_key);
    printf("TUTK_SDK_Set_License_Key: %d\n", ret);
    if (ret < 0) {
        return -1;
    }

    strcpy(gUdid, argv[2]);
    fp = fopen(argv[3], "r");
    fseek(fp, 0, SEEK_END);
    long fsize = ftell(fp);
    fseek(fp, 0, SEEK_SET);

    char *profile = malloc(fsize + 1);
    fread(profile, 1, fsize, fp);
    fclose(fp);
    profile[fsize] = 0;

    ret = loadDisposableParams(gPinCode, gSecretId, gPsk);
    if(ret < 0) {
        printf("loadDisposableParams()=%d\n", ret);
        return -1;
    }

    LogAttr log_attr;
    memset(&log_attr, 0, sizeof(log_attr));
    char log_path[] = "./device_log.txt";
    log_attr.path = log_path;

    ret = IOTC_Set_Log_Attr(log_attr);
    printf("IOTC_Set_Log_Attr ret[%d]\n", ret);

	printf("IOTCAPIs_Device start...\n");
	ret = IOTC_Initialize2(0);
	printf("IOTC_Initialize() ret = %d\n", ret);
	if(ret != IOTC_ER_NoERROR) {
		printf("IOTCAPIs_Device exit...!!\n");
		return -1;
	}

	ret = Nebula_Initialize();
    printf("Nebula_Initialize ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Device exit...!!\n");
        return -1;
    }

    TUTKJsonObject *profile_obj = NULL;
    TUTK_Json_Obj_Create_From_String(profile, &profile_obj);
    const char *profile_str = TUTK_Json_Obj_To_String(profile_obj);
    ret = Nebula_Device_New(gUdid, gSecretId, profile_str, command_handle, identity_handle, settings_change_handle, &device_ctx);
    printf("Nebula_Device_New ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Device exit...!!\n");
        return -1;
    }

    TUTK_Json_Obj_Release(profile_obj);
    free(profile);

    ret = Nebula_Device_Login(device_ctx, device_login_state_handle);
    printf("Nebula_Device_Login ret[%d]\n", ret);
    if (ret != NEBULA_ER_NoERROR) {
        printf("IOTCAPIs_Device exit...!!\n");
        return -1;
    }

    while(1)
    {
        ret = Nebula_Device_Bind(device_ctx, gPinCode, gPsk, 60000, &gBindAbort);
        printf("Nebula_Device_Bind ret[%d]\n", ret);
        if (ret == NEBULA_ER_NoERROR)
            break;
        else if (ret != NEBULA_ER_TIMEOUT) {
            printf("IOTCAPIs_Device exit...!!\n");
            return -1;
        }
    }

    ret = IOTC_Device_Login_By_Nebula(device_ctx);
    printf("IOTC_Device_Login_By_Nebula() ret = %d\n", ret);
    if (ret < 0) {
        printf("IOTCAPIs_Device exit...!!\n");
        return -1;
    }

    while(1) {
        ret = IOTC_Device_Listen_By_Nebula(device_ctx, 1000);
        if (ret >= 0){
            printf("Session[%d] connect!\n", ret);
            struct st_SInfoEx Sinfo;
            Sinfo.size = sizeof(struct st_SInfoEx);
            if(IOTC_Session_Check_Ex(ret, &Sinfo) == IOTC_ER_NoERROR)
            {
                char *mode[3] = {"P2P", "RLY", "LAN"};
                if( isdigit( Sinfo.RemoteIP[0] ))
                    printf("Client is from[IP:%s, Port:%d] Mode[%s] VPG[%d:%d:%d] VER[%X] NAT[%d] AES[%d]\n", Sinfo.RemoteIP, Sinfo.RemotePort, mode[(int)Sinfo.Mode], Sinfo.VID, Sinfo.PID, Sinfo.GID, Sinfo.IOTCVersion, Sinfo.RemoteNatType, Sinfo.isSecure);
            }
        } else if (ret != IOTC_ER_TIMEOUT){
            printf("IOTC_Device_Listen_By_Nebula ret[%d]\n", ret);
            break;
        }
    }

    ret = Nebula_Device_Delete(device_ctx);

    Nebula_DeInitialize();
	IOTC_DeInitialize();
    printf("IOTC_DeInitialize()\n");
	return 0;
}

